--
-- CREATE_TABLE
--

--
-- CLASS DEFINITIONS
--
CREATE TABLE hobbies_r (
	name		text, 
	person 		text
);

CREATE TABLE equipment_r (
	name 		text,
	hobby		text
);

CREATE TABLE onek (
	unique1		int4,
	unique2		int4,
	two			int4,
	four		int4,
	ten			int4,
	twenty		int4,
	hundred		int4,
	thousand	int4,
	twothousand	int4,
	fivethous	int4,
	tenthous	int4,
	odd			int4,
	even		int4,
	stringu1	name,
	stringu2	name,
	string4		name
);

CREATE TABLE tenk1 (
	unique1		int4,
	unique2		int4,
	two			int4,
	four		int4,
	ten			int4,
	twenty		int4,
	hundred		int4,
	thousand	int4,
	twothousand	int4,
	fivethous	int4,
	tenthous	int4,
	odd			int4,
	even		int4,
	stringu1	name,
	stringu2	name,
	string4		name
) WITH OIDS;

CREATE TABLE tenk2 (
	unique1 	int4,
	unique2 	int4,
	two 	 	int4,
	four 		int4,
	ten			int4,
	twenty 		int4,
	hundred 	int4,
	thousand 	int4,
	twothousand int4,
	fivethous 	int4,
	tenthous	int4,
	odd			int4,
	even		int4,
	stringu1	name,
	stringu2	name,
	string4		name
);


CREATE TABLE person (
	name 		text,
	age			int4,
	location 	point
);


CREATE TABLE emp (
	salary 		int4,
	manager 	name
) INHERITS (person) WITH OIDS;


CREATE TABLE student (
	gpa 		float8
) INHERITS (person);


CREATE TABLE stud_emp (
	percent 	int4
) INHERITS (emp, student);


CREATE TABLE city (
	name		name,
	location 	box,
	budget 		city_budget
);

CREATE TABLE dept (
	dname		name,
	mgrname 	text
);

CREATE TABLE slow_emp4000 (
	home_base	 box
);

CREATE TABLE fast_emp4000 (
	home_base	 box
);

CREATE TABLE road (
	name		text,
	thepath 	path
);

CREATE TABLE ihighway () INHERITS (road);

CREATE TABLE shighway (
	surface		text
) INHERITS (road);

CREATE TABLE real_city (
	pop			int4,
	cname		text,
	outline 	path
);

--
-- test the "star" operators a bit more thoroughly -- this time,
-- throw in lots of NULL fields...
--
-- a is the type root
-- b and c inherit from a (one-level single inheritance)
-- d inherits from b and c (two-level multiple inheritance)
-- e inherits from c (two-level single inheritance)
-- f inherits from e (three-level single inheritance)
--
CREATE TABLE a_star (
	class		char, 
	a 			int4
);

CREATE TABLE b_star (
	b 			text
) INHERITS (a_star);

CREATE TABLE c_star (
	c 			name
) INHERITS (a_star);

CREATE TABLE d_star (
	d 			float8
) INHERITS (b_star, c_star);

CREATE TABLE e_star (
	e 			int2
) INHERITS (c_star);

CREATE TABLE f_star (
	f 			polygon
) INHERITS (e_star);

CREATE TABLE aggtest (
	a 			int2,
	b			float4
);

CREATE TABLE hash_i4_heap (
	seqno 		int4,
	random 		int4
) distributed by (seqno);

CREATE TABLE hash_name_heap (
	seqno 		int4,
	random 		name
) distributed by (seqno);

CREATE TABLE hash_txt_heap (
	seqno 		int4,
	random 		text
) distributed by (seqno);

CREATE TABLE hash_f8_heap (
	seqno		int4,
	random 		float8
) distributed by (seqno);

-- don't include the hash_ovfl_heap stuff in the distribution
-- the data set is too large for what it's worth
-- 
-- CREATE TABLE hash_ovfl_heap (
--	x			int4,
--	y			int4
-- );

CREATE TABLE bt_i4_heap (
	seqno 		int4,
	random 		int4
);

CREATE TABLE bt_name_heap (
	seqno 		name,
	random 		int4
);

CREATE TABLE bt_txt_heap (
	seqno 		text,
	random 		int4
);

CREATE TABLE bt_f8_heap (
	seqno 		float8, 
	random 		int4
);

CREATE TABLE array_op_test (
	seqno		int4,
	i			int4[],
	t			text[]
);

CREATE TABLE array_index_op_test (
	seqno		int4,
	i			int4[],
	t			text[]
);

CREATE TABLE test_tsvector( 
	t text, 
	a tsvector 
);

-- large custom format table definitions
drop external table if exists large_custom_format_definitions;
create readable external table large_custom_format_definitions (
    address varchar(46),
    state varchar(2),
    zipcode int,
    id int,
    id_last_two int,
    id_last_three int,
    hhid int,
    efxid int,
    riskscore int,
    cid varchar(18),
    flag varchar(1),
    census_tract int,
    census_block int,
    census_code int,
    address_flag varchar(1),
    cust_attr1 int,
    cust_attr2 int,
    cust_attr3 int,
    cust_attr4 int,
    cust_attr5 varchar(1),
    cust_attr6 varchar(1),
    cust_attr7 int,
    cust_attr8 int,
    cust_attr9 int,
    cust_attr10 int,
    cust_attr11 varchar(2),
    cust_attr12 varchar(1),
    cust_attr13 int,
    cust_attr14 int,
    cust_attr15 int,
    cust_attr16 int,
    cust_attr17 varchar(1),
    cust_attr18 varchar(1),
    cust_attr19 int,
    cust_attr20 int,
    cust_attr21 int,
    cust_attr22 int,
    cust_attr23 varchar(2),
    cust_attr24 varchar(1),
    cust_attr25 int,
    cust_attr26 int,
    cust_attr27 int,
    cust_attr28 int,
    cust_attr29 varchar(1),
    cust_attr30 varchar(1),
    cust_attr31 int,
    cust_attr32 int,
    cust_attr33 int,
    cust_attr34 int,
    cust_attr35 varchar(2),
    cust_attr36 varchar(1),
    cust_attr37 int,
    cust_attr38 int,
    cust_attr39 int,
    cust_attr40 int,
    cust_attr41 varchar(1),
    cust_attr42 varchar(1),
    cust_attr43 int,
    cust_attr44 int,
    cust_attr45 int,
    cust_attr46 int,
    cust_attr47 varchar(2),
    cust_attr48 varchar(1),
    cust_attr49 int,
    cust_attr50 int,
    cust_attr51 int,
    cust_attr52 int,
    cust_attr53 varchar(1),
    cust_attr54 varchar(1),
    cust_attr55 int,
    cust_attr56 int,
    cust_attr57 int,
    cust_attr58 int,
    cust_attr59 varchar(2),
    cust_attr60 varchar(1),
    cust_attr61 int,
    cust_attr62 int,
    cust_attr63 int,
    cust_attr64 int,
    cust_attr65 varchar(1),
    cust_attr66 varchar(1),
    cust_attr67 int,
    cust_attr68 int,
    cust_attr69 int,
    cust_attr70 int,
    cust_attr71 varchar(2),
    cust_attr72 varchar(1),
    cust_attr73 int,
    cust_attr74 int,
    cust_attr75 int,
    cust_attr76 int,
    cust_attr77 varchar(1),
    cust_attr78 varchar(1),
    cust_attr79 int,
    cust_attr80 int,
    cust_attr81 int,
    cust_attr82 int,
    cust_attr83 varchar(2),
    cust_attr84 varchar(1),
    cust_attr85 int,
    cust_attr86 int,
    cust_attr87 int,
    cust_attr88 int,
    cust_attr89 varchar(1),
    cust_attr90 varchar(1),
    cust_attr91 int,
    cust_attr92 int,
    cust_attr93 int,
    cust_attr94 int,
    cust_attr95 varchar(2),
    cust_attr96 varchar(1),
    cust_attr97 int,
    cust_attr98 int,
    cust_attr99 int,
    cust_attr100 int,
    cust_attr101 varchar(1),
    cust_attr102 varchar(1),
    cust_attr103 int,
    cust_attr104 int,
    cust_attr105 int,
    cust_attr106 int,
    cust_attr107 varchar(2),
    cust_attr108 varchar(1),
    cust_attr109 int,
    cust_attr110 int,
    cust_attr111 int,
    cust_attr112 int,
    cust_attr113 varchar(1),
    cust_attr114 varchar(1),
    cust_attr115 int,
    cust_attr116 int,
    cust_attr117 int,
    cust_attr118 int,
    cust_attr119 varchar(2),
    cust_attr120 varchar(1),
    cust_attr121 int,
    cust_attr122 int,
    cust_attr123 int,
    cust_attr124 int,
    cust_attr125 varchar(1),
    cust_attr126 varchar(1),
    cust_attr127 int,
    cust_attr128 int,
    cust_attr129 int,
    cust_attr130 int,
    cust_attr131 varchar(2),
    cust_attr132 varchar(1),
    cust_attr133 int,
    cust_attr134 int,
    cust_attr135 int,
    cust_attr136 int,
    cust_attr137 varchar(1),
    cust_attr138 varchar(1),
    cust_attr139 int,
    cust_attr140 int,
    cust_attr141 int,
    cust_attr142 int,
    cust_attr143 varchar(2),
    cust_attr144 varchar(1),
    cust_attr145 int,
    cust_attr146 int,
    cust_attr147 int,
    cust_attr148 int,
    cust_attr149 varchar(1),
    cust_attr150 varchar(1),
    cust_attr151 int,
    cust_attr152 int,
    cust_attr153 int,
    cust_attr154 int,
    cust_attr155 varchar(2),
    cust_attr156 varchar(1),
    cust_attr157 int,
    cust_attr158 int,
    cust_attr159 int,
    cust_attr160 int,
    cust_attr161 varchar(1),
    cust_attr162 varchar(1),
    cust_attr163 int,
    cust_attr164 int,
    cust_attr165 int,
    cust_attr166 int,
    cust_attr167 varchar(2),
    cust_attr168 varchar(1),
    cust_attr169 int,
    cust_attr170 int,
    cust_attr171 int,
    cust_attr172 int,
    cust_attr173 varchar(1),
    cust_attr174 varchar(1),
    cust_attr175 int,
    cust_attr176 int,
    cust_attr177 int,
    cust_attr178 int,
    cust_attr179 varchar(2),
    cust_attr180 varchar(1),
    cust_attr181 int,
    cust_attr182 int,
    cust_attr183 int,
    cust_attr184 int,
    cust_attr185 varchar(1),
    cust_attr186 varchar(1),
    cust_attr187 int,
    cust_attr188 int,
    cust_attr189 int,
    cust_attr190 int,
    cust_attr191 varchar(2),
    cust_attr192 varchar(1),
    cust_attr193 int,
    cust_attr194 int,
    cust_attr195 int,
    cust_attr196 int,
    cust_attr197 varchar(1),
    cust_attr198 varchar(1),
    cust_attr199 int,
    cust_attr200 int,
    cust_attr201 int,
    cust_attr202 int,
    cust_attr203 varchar(2),
    cust_attr204 varchar(1),
    cust_attr205 int,
    cust_attr206 int,
    cust_attr207 int,
    cust_attr208 int,
    cust_attr209 varchar(1),
    cust_attr210 varchar(1),
    cust_attr211 int,
    cust_attr212 int,
    cust_attr213 int,
    cust_attr214 int,
    cust_attr215 varchar(2),
    cust_attr216 varchar(1),
    cust_attr217 int,
    cust_attr218 int,
    cust_attr219 int,
    cust_attr220 int,
    cust_attr221 varchar(1),
    cust_attr222 varchar(1),
    cust_attr223 int,
    cust_attr224 int,
    cust_attr225 int,
    cust_attr226 int,
    cust_attr227 varchar(2),
    cust_attr228 varchar(1),
    cust_attr229 int,
    cust_attr230 int,
    cust_attr231 int,
    cust_attr232 int,
    cust_attr233 varchar(1),
    cust_attr234 varchar(1),
    cust_attr235 int,
    cust_attr236 int,
    cust_attr237 int,
    cust_attr238 int,
    cust_attr239 varchar(2),
    cust_attr240 varchar(1),
    cust_attr241 int,
    cust_attr242 int,
    cust_attr243 int,
    cust_attr244 int,
    cust_attr245 varchar(1),
    cust_attr246 varchar(1),
    cust_attr247 int,
    cust_attr248 int,
    cust_attr249 int,
    cust_attr250 int,
    cust_attr251 varchar(2),
    cust_attr252 varchar(1),
    cust_attr253 int,
    cust_attr254 int,
    cust_attr255 int,
    cust_attr256 int,
    cust_attr257 varchar(1),
    cust_attr258 varchar(1),
    cust_attr259 int,
    cust_attr260 int,
    cust_attr261 int,
    cust_attr262 int,
    cust_attr263 varchar(2),
    cust_attr264 varchar(1),
    cust_attr265 int,
    cust_attr266 int,
    cust_attr267 int,
    cust_attr268 int,
    cust_attr269 varchar(1),
    cust_attr270 varchar(1),
    cust_attr271 int,
    cust_attr272 int,
    cust_attr273 int,
    cust_attr274 int,
    cust_attr275 varchar(2),
    cust_attr276 varchar(1),
    cust_attr277 int,
    cust_attr278 int,
    cust_attr279 int,
    cust_attr280 int,
    cust_attr281 varchar(1),
    cust_attr282 varchar(1),
    cust_attr283 int,
    cust_attr284 int,
    cust_attr285 int,
    cust_attr286 int,
    cust_attr287 varchar(2),
    cust_attr288 varchar(1),
    cust_attr289 int,
    cust_attr290 varchar(1),
    cust_attr291 varchar(1),
    cust_attr292 varchar(1),
    cust_attr293 int,
    cust_attr294 int,
    cust_attr295 int,
    cust_attr296 int,
    cust_attr297 int,
    cust_attr298 int,
    cust_attr299 int,
    cust_attr300 int,
    cust_attr301 int,
    cust_attr302 int,
    cust_attr303 int,
    cust_attr304 int,
    cust_attr305 int,
    cust_attr306 int,
    cust_attr307 int,
    cust_attr308 int,
    cust_attr309 int,
    cust_attr310 int,
    cust_attr311 int,
    cust_attr312 int,
    cust_attr313 int,
    cust_attr314 int,
    cust_attr315 int,
    cust_attr316 int,
    cust_attr317 int,
    cust_attr318 int,
    cust_attr319 int,
    cust_attr320 int,
    cust_attr321 int,
    cust_attr322 int,
    cust_attr323 int,
    cust_attr324 int,
    cust_attr325 int,
    cust_attr326 int,
    cust_attr327 int,
    cust_attr328 int,
    cust_attr329 int,
    cust_attr330 int,
    cust_attr331 int,
    cust_attr332 int,
    cust_attr333 int,
    cust_attr334 int,
    cust_attr335 int,
    cust_attr336 int,
    cust_attr337 int,
    cust_attr338 int,
    cust_attr339 int,
    cust_attr340 int,
    cust_attr341 varchar(1),
    crtr_attr1 int,
    crtr_attr2 int,
    crtr_attr3 int,
    crtr_attr4 int,
    crtr_attr5 int,
    crtr_attr6 int,
    crtr_attr7 int,
    crtr_attr8 int,
    crtr_attr9 int,
    crtr_attr10 int,
    crtr_attr11 int,
    crtr_attr12 int,
    crtr_attr13 int,
    crtr_attr14 int,
    crtr_attr15 int,
    crtr_attr16 int,
    crtr_attr17 int,
    crtr_attr18 int,
    crtr_attr19 int,
    crtr_attr20 int,
    crtr_attr21 int,
    crtr_attr22 int,
    crtr_attr23 int,
    crtr_attr24 int,
    crtr_attr25 int,
    crtr_attr26 int,
    crtr_attr27 int,
    crtr_attr28 int,
    crtr_attr29 int,
    crtr_attr30 int,
    crtr_attr31 int,
    crtr_attr32 int,
    crtr_attr33 int,
    crtr_attr34 int,
    crtr_attr35 int,
    crtr_attr36 int,
    crtr_attr37 int,
    crtr_attr38 int,
    crtr_attr39 int,
    crtr_attr40 int,
    crtr_attr41 int,
    crtr_attr42 int,
    crtr_attr43 int,
    crtr_attr44 int,
    crtr_attr45 int,
    crtr_attr46 int,
    crtr_attr47 int,
    crtr_attr48 int,
    crtr_attr49 int,
    crtr_attr50 int,
    crtr_attr51 int,
    crtr_attr52 int,
    crtr_attr53 int,
    crtr_attr54 int,
    crtr_attr55 int,
    crtr_attr56 int,
    crtr_attr57 int,
    crtr_attr58 int,
    crtr_attr59 int,
    crtr_attr60 int,
    crtr_attr61 int,
    crtr_attr62 int,
    crtr_attr63 int,
    crtr_attr64 int,
    crtr_attr65 int,
    crtr_attr66 int,
    crtr_attr67 int,
    crtr_attr68 int,
    crtr_attr69 int,
    crtr_attr70 int,
    crtr_attr71 int,
    crtr_attr72 int,
    crtr_attr73 int,
    crtr_attr74 int,
    crtr_attr75 int,
    crtr_attr76 int,
    crtr_attr77 int,
    crtr_attr78 int,
    crtr_attr79 int,
    crtr_attr80 int,
    crtr_attr81 int,
    crtr_attr82 int,
    crtr_attr83 int,
    crtr_attr84 int,
    crtr_attr85 int,
    crtr_attr86 int,
    crtr_attr87 int,
    crtr_attr88 int,
    crtr_attr89 int,
    crtr_attr90 int,
    crtr_attr91 int,
    crtr_attr92 int,
    crtr_attr93 int,
    crtr_attr94 int,
    crtr_attr95 int,
    crtr_attr96 int,
    crtr_attr97 int,
    crtr_attr98 int,
    crtr_attr99 int,
    crtr_attr100 int,
    crtr_attr101 int,
    crtr_attr102 int,
    crtr_attr103 int,
    crtr_attr104 int,
    crtr_attr105 int,
    crtr_attr106 int,
    crtr_attr107 int,
    crtr_attr108 int,
    crtr_attr109 int,
    crtr_attr110 int,
    crtr_attr111 int,
    crtr_attr112 int,
    crtr_attr113 int,
    crtr_attr114 int,
    crtr_attr115 int,
    crtr_attr116 int,
    crtr_attr117 int,
    crtr_attr118 int,
    crtr_attr119 int,
    crtr_attr120 int,
    crtr_attr121 int,
    crtr_attr122 int,
    crtr_attr123 int,
    crtr_attr124 int,
    crtr_attr125 int,
    crtr_attr126 int,
    crtr_attr127 int,
    crtr_attr128 int,
    crtr_attr129 int,
    crtr_attr130 int,
    crtr_attr131 int,
    crtr_attr132 int,
    crtr_attr133 int,
    crtr_attr134 int,
    crtr_attr135 int,
    crtr_attr136 int,
    crtr_attr137 int,
    crtr_attr138 int,
    crtr_attr139 int,
    crtr_attr140 int,
    crtr_attr141 int,
    crtr_attr142 int,
    crtr_attr143 int,
    crtr_attr144 int,
    crtr_attr145 int,
    crtr_attr146 int,
    crtr_attr147 int,
    crtr_attr148 int,
    crtr_attr149 int,
    crtr_attr150 int,
    crtr_attr151 int,
    crtr_attr152 int,
    crtr_attr153 int,
    crtr_attr154 int,
    crtr_attr155 int,
    crtr_attr156 int,
    crtr_attr157 int,
    crtr_attr158 int,
    crtr_attr159 int,
    crtr_attr160 int,
    crtr_attr161 int,
    crtr_attr162 int,
    crtr_attr163 int,
    crtr_attr164 int,
    crtr_attr165 int,
    crtr_attr166 int,
    crtr_attr167 int,
    crtr_attr168 int,
    crtr_attr169 int,
    crtr_attr170 int,
    crtr_attr171 int,
    crtr_attr172 int,
    crtr_attr173 int,
    crtr_attr174 int,
    crtr_attr175 int,
    crtr_attr176 int,
    crtr_attr177 int,
    crtr_attr178 int,
    crtr_attr179 int,
    crtr_attr180 int,
    crtr_attr181 int,
    crtr_attr182 int,
    crtr_attr183 int,
    crtr_attr184 int,
    crtr_attr185 int,
    crtr_attr186 int,
    crtr_attr187 int,
    cma_attr3746 int,
    cma_attr3000 int,
    cma_attr3001 int,
    cma_attr3002 int,
    cma_attr3100 int,
    cma_attr3102 int,
    cma_attr3104 int,
    cma_attr3105 int,
    cma_attr3109 int,
    cma_attr3111 int,
    cma_attr3113 int,
    cma_attr3116 int,
    cma_attr3117 int,
    cma_attr3119 int,
    cma_attr3120 int,
    cma_attr3122 int,
    cma_attr3123 int,
    cma_attr3124 int,
    cma_attr3126 int,
    cma_attr3127 int,
    cma_attr3131 int,
    cma_attr3132 int,
    cma_attr3133 int,
    cma_attr3134 int,
    cma_attr3135 int,
    cma_attr3136 int,
    cma_attr3137 int,
    cma_attr3138 int,
    cma_attr3139 int,
    cma_attr3140 int,
    cma_attr3141 int,
    cma_attr3142 int,
    cma_attr3143 int,
    cma_attr3144 int,
    cma_attr3145 int,
    cma_attr3146 int,
    cma_attr3147 int,
    cma_attr3148 int,
    cma_attr3150 int,
    cma_attr3152 int,
    cma_attr3153 int,
    cma_attr3157 int,
    cma_attr3159 int,
    cma_attr3160 int,
    cma_attr3161 int,
    cma_attr3162 int,
    cma_attr3163 int,
    cma_attr3164 int,
    cma_attr3165 int,
    cma_attr3166 int,
    cma_attr3167 int,
    cma_attr3168 int,
    cma_attr3169 int,
    cma_attr3203 int,
    cma_attr3204 int,
    cma_attr3205 int,
    cma_attr3206 int,
    cma_attr3207 int,
    cma_attr3208 int,
    cma_attr3215 int,
    cma_attr3217 int,
    cma_attr3219 int,
    cma_attr3223 int,
    cma_attr3224 int,
    cma_attr3228 int,
    cma_attr3231 int,
    cma_attr3234 int,
    cma_attr3235 int,
    cma_attr3236 int,
    cma_attr3237 int,
    cma_attr3239 int,
    cma_attr3242 int,
    cma_attr3245 int,
    cma_attr3246 int,
    cma_attr3257 int,
    cma_attr3266 int,
    cma_attr3268 int,
    cma_attr3270 int,
    cma_attr3272 int,
    cma_attr3273 int,
    cma_attr3276 int,
    cma_attr3277 int,
    cma_attr3285 int,
    cma_attr3286 int,
    cma_attr3288 int,
    cma_attr3290 int,
    cma_attr3292 int,
    cma_attr3293 int,
    cma_attr3296 int,
    cma_attr3297 int,
    cma_attr3307 int,
    cma_attr3309 int,
    cma_attr3311 int,
    cma_attr3312 int,
    cma_attr3315 int,
    cma_attr3316 int,
    cma_attr3318 int,
    cma_attr3320 int,
    cma_attr3322 int,
    cma_attr3323 int,
    cma_attr3326 int,
    cma_attr3327 int,
    cma_attr3329 int,
    cma_attr3330 int,
    cma_attr3332 int,
    cma_attr3333 int,
    cma_attr3335 int,
    cma_attr3336 int,
    cma_attr3338 int,
    cma_attr3340 int,
    cma_attr3347 int,
    cma_attr3368 int,
    cma_attr3370 int,
    cma_attr3376 int,
    cma_attr3379 int,
    cma_attr3380 int,
    cma_attr3382 int,
    cma_attr3383 int,
    cma_attr3385 int,
    cma_attr3386 int,
    cma_attr3387 int,
    cma_attr3388 int,
    cma_attr3389 int,
    cma_attr3391 int,
    cma_attr3392 int,
    cma_attr3394 int,
    cma_attr3395 int,
    cma_attr3396 int,
    cma_attr3397 int,
    cma_attr3398 int,
    cma_attr3400 int,
    cma_attr3401 int,
    cma_attr3403 int,
    cma_attr3404 int,
    cma_attr3405 int,
    cma_attr3406 int,
    cma_attr3407 int,
    cma_attr3409 int,
    cma_attr3410 int,
    cma_attr3412 int,
    cma_attr3413 int,
    cma_attr3414 int,
    cma_attr3415 int,
    cma_attr3417 int,
    cma_attr3419 int,
    cma_attr3420 int,
    cma_attr3422 int,
    cma_attr3423 int,
    cma_attr3424 int,
    cma_attr3425 int,
    cma_attr3426 int,
    cma_attr3428 int,
    cma_attr3430 int,
    cma_attr3431 int,
    cma_attr3433 int,
    cma_attr3434 int,
    cma_attr3435 int,
    cma_attr3436 int,
    cma_attr3437 int,
    cma_attr3439 int,
    cma_attr3441 int,
    cma_attr3442 int,
    cma_attr3444 int,
    cma_attr3445 int,
    cma_attr3446 int,
    cma_attr3447 int,
    cma_attr3448 int,
    cma_attr3450 int,
    cma_attr3452 int,
    cma_attr3453 int,
    cma_attr3455 int,
    cma_attr3456 int,
    cma_attr3457 int,
    cma_attr3458 int,
    cma_attr3535 int,
    cma_attr3536 int,
    cma_attr3537 int,
    cma_attr3539 int,
    cma_attr3540 int,
    cma_attr3544 int,
    cma_attr3546 int,
    cma_attr3547 int,
    cma_attr3548 int,
    cma_attr3550 int,
    cma_attr3551 int,
    cma_attr3555 int,
    cma_attr3557 int,
    cma_attr3558 int,
    cma_attr3559 int,
    cma_attr3561 int,
    cma_attr3562 int,
    cma_attr3566 int,
    cma_attr3568 int,
    cma_attr3569 int,
    cma_attr3570 int,
    cma_attr3572 int,
    cma_attr3573 int,
    cma_attr3577 int,
    cma_attr3579 int,
    cma_attr3580 int,
    cma_attr3592 int,
    cma_attr3593 int,
    cma_attr3594 int,
    cma_attr3596 int,
    cma_attr3597 int,
    cma_attr3598 int,
    cma_attr3600 int,
    cma_attr3601 int,
    cma_attr3602 int,
    cma_attr3603 int,
    cma_attr3605 int,
    cma_attr3607 int,
    cma_attr3608 int,
    cma_attr3611 int,
    cma_attr3612 int,
    cma_attr3723 int,
    cma_attr3724 int,
    cma_attr3725 int,
    cma_attr3726 int,
    cma_attr3727 int,
    cma_attr3728 int,
    cma_attr3729 int,
    cma_attr3900 int,
    cma_attr3901 int,
    cma_attr3902 varchar(1),
    cma_attr3903 varchar(1),
    cma_attr3904 varchar(1),
    cma_attr3905 varchar(1),
    cma_attr3907 int,
    cma_attr3908 int,
    cma_attr3909 int,
    cma_attr3911 int,
    cma_attr3912 int,
    cma_attr3913 int,
    cma_attr3747 int,
    cma_attr3748 int,
    cma_attr3749 int,
    cma_attr3750 int,
    cma_attr3751 int,
    cma_attr3752 int,
    cma_attr3753 int,
    cma_attr3754 int,
    cma_attr3812 int,
    cma_attr3813 int,
    cma_attr3993 int,
    cma_attr3994 int,
    cma_attr3995 int,
    cma_attr3825 int,
    cma_attr3826 int,
    cma_attr3827 int,
    cma_attr3828 int,
    cma_attr3829 int,
    cma_attr3830 int,
    cma_attr3831 int,
    cma_attr3832 int,
    cma_attr3833 int,
    cma_attr3834 int,
    cma_attr3835 int,
    cma_attr3836 int,
    cma_attr3837 int,
    cma_attr3838 int,
    cma_attr3839 int,
    cma_attr3840 int,
    cma_attr3841 int,
    cma_attr3842 int,
    cma_attr3843 int,
    cma_attr3844 int,
    cma_attr3845 int,
    cma_attr3854 int,
    cma_attr3855 int,
    cma_attr3856 int,
    cma_attr3857 int,
    cma_attr3858 int,
    cma_attr3859 int,
    cma_attr3860 int,
    cma_attr3861 int,
    cma_attr3862 int,
    cma_attr3863 int,
    cma_attr3864 int,
    cma_attr3865 int,
    cma_attr3866 int,
    cma_attr3867 int,
    cma_attr3868 int,
    cma_attr3869 int,
    cma_attr3938 int,
    cma_attr3939 int,
    cma_attr3940 int,
    cma_attr3870 int,
    cma_attr3871 int,
    cma_attr3872 int,
    cma_attr3873 int,
    cma_attr3874 int,
    cma_attr3875 int,
    cma_attr3876 int,
    cma_attr3877 int,
    cma_attr3878 int,
    cma_attr3879 int,
    cma_attr3880 int,
    cma_attr3881 int,
    cma_attr3882 int,
    cma_attr3883 int,
    cma_attr3884 int,
    cma_attr3885 int,
    cma_attr3917 int,
    cma_attr3918 int,
    cma_attr3919 int,
    cma_attr3920 int,
    cma_attr3921 int,
    cma_attr3922 int,
    cma_attr3923 int,
    cma_attr3924 int,
    cma_attr3925 int,
    cma_attr3926 int,
    cma_attr3927 int,
    cma_attr3928 int,
    cma_attr3929 int,
    cma_attr3930 int,
    cma_attr3931 int,
    cma_attr3932 int,
    cma_attr3933 int,
    cma_attr3886 int,
    cma_attr3934 int,
    cma_attr3887 int,
    cma_attr3894 int,
    cma_attr3895 int,
    cma_attr3896 int,
    cma_attr3897 int,
    cma_attr3898 int,
    cma_attr3899 int,
    cma_attr3914 int,
    cma_attr3915 int,
    cma_attr3916 int,
    cma_attr3941 int,
    cma_attr3942 int,
    cma_attr3943 int,
    cma_attr3944 int,
    cma_attr3945 int,
    cma_attr3946 int,
    cma_attr3947 int,
    cma_attr3948 int,
    cma_attr3949 int,
    cma_attr3950 int,
    cma_attr3951 int,
    cma_attr3952 int,
    cma_attr3953 int,
    cma_attr3954 int,
    cma_attr3955 int,
    cma_attr3956 int,
    cma_attr3957 int,
    cma_attr3958 int,
    cma_attr3959 int,
    cma_attr3960 int,
    cma_attr3961 int,
    cma_attr3962 int,
    cma_attr3963 int,
    cma_attr3964 int,
    cma_attr3968 int,
    cma_attr3969 int,
    cma_attr3970 int,
    cma_attr3971 int,
    cma_attr3973 int,
    cma_attr3974 int,
    cma_attr3975 int,
    cma_attr3976 int,
    cma_attr3978 int,
    cma_attr3979 int,
    cma_attr3980 int,
    cma_attr3981 int,
    cma_attr3983 int,
    cma_attr3984 int,
    cma_attr3985 int,
    cma_attr3986 int,
    cma_attr3987 int,
    census_tract_2010 int,
    census_block_2010 int,
    county_code_2010 int
)
LOCATION ('file://localhost/data')
FORMAT 'CUSTOM' (formatter=fixedwidth_in,
address='46',
unused='1',
state='2',
unused1='1',
zipcode='5',
unused2='1',
id='9',
unused3='1',
hhid='11',
unused4='1',
efxid='11',
unused5='1',
riskscore='3',
unused6='1',
cid='18',
unused7='1',
flag='1',
unused8='1',
census_tract='6',
unused9='1',
census_block='4',
unused10='1',
census_code='3',
unused11='1',
address_flag='1',
unused12='1',
cust_attr1='4',
cust_attr2='6',
cust_attr3='7',
cust_attr4='7',
cust_attr5='1',
cust_attr6='1',
cust_attr7='3',
cust_attr8='7',
cust_attr9='3',
cust_attr10='3',
cust_attr11='2',
cust_attr12='1',
cust_attr13='6',
cust_attr14='6',
cust_attr15='7',
cust_attr16='7',
cust_attr17='1',
cust_attr18='1',
cust_attr19='3',
cust_attr20='7',
cust_attr21='3',
cust_attr22='3',
cust_attr23='2',
cust_attr24='1',
cust_attr25='6',
cust_attr26='6',
cust_attr27='7',
cust_attr28='7',
cust_attr29='1',
cust_attr30='1',
cust_attr31='3',
cust_attr32='7',
cust_attr33='3',
cust_attr34='3',
cust_attr35='2',
cust_attr36='1',
cust_attr37='6',
cust_attr38='6',
cust_attr39='7',
cust_attr40='7',
cust_attr41='1',
cust_attr42='1',
cust_attr43='3',
cust_attr44='7',
cust_attr45='3',
cust_attr46='3',
cust_attr47='2',
cust_attr48='1',
cust_attr49='6',
cust_attr50='6',
cust_attr51='7',
cust_attr52='7',
cust_attr53='1',
cust_attr54='1',
cust_attr55='3',
cust_attr56='7',
cust_attr57='3',
cust_attr58='3',
cust_attr59='2',
cust_attr60='1',
cust_attr61='6',
cust_attr62='6',
cust_attr63='7',
cust_attr64='7',
cust_attr65='1',
cust_attr66='1',
cust_attr67='3',
cust_attr68='7',
cust_attr69='3',
cust_attr70='3',
cust_attr71='2',
cust_attr72='1',
cust_attr73='6',
cust_attr74='6',
cust_attr75='7',
cust_attr76='7',
cust_attr77='1',
cust_attr78='1',
cust_attr79='3',
cust_attr80='7',
cust_attr81='3',
cust_attr82='3',
cust_attr83='2',
cust_attr84='1',
cust_attr85='6',
cust_attr86='6',
cust_attr87='7',
cust_attr88='7',
cust_attr89='1',
cust_attr90='1',
cust_attr91='3',
cust_attr92='7',
cust_attr93='3',
cust_attr94='3',
cust_attr95='2',
cust_attr96='1',
cust_attr97='6',
cust_attr98='6',
cust_attr99='7',
cust_attr100='7',
cust_attr101='1',
cust_attr102='1',
cust_attr103='3',
cust_attr104='7',
cust_attr105='3',
cust_attr106='3',
cust_attr107='2',
cust_attr108='1',
cust_attr109='6',
cust_attr110='6',
cust_attr111='7',
cust_attr112='7',
cust_attr113='1',
cust_attr114='1',
cust_attr115='3',
cust_attr116='7',
cust_attr117='3',
cust_attr118='3',
cust_attr119='2',
cust_attr120='1',
cust_attr121='6',
cust_attr122='6',
cust_attr123='7',
cust_attr124='7',
cust_attr125='1',
cust_attr126='1',
cust_attr127='3',
cust_attr128='7',
cust_attr129='3',
cust_attr130='3',
cust_attr131='2',
cust_attr132='1',
cust_attr133='6',
cust_attr134='6',
cust_attr135='7',
cust_attr136='7',
cust_attr137='1',
cust_attr138='1',
cust_attr139='3',
cust_attr140='7',
cust_attr141='3',
cust_attr142='3',
cust_attr143='2',
cust_attr144='1',
cust_attr145='6',
cust_attr146='6',
cust_attr147='7',
cust_attr148='7',
cust_attr149='1',
cust_attr150='1',
cust_attr151='3',
cust_attr152='7',
cust_attr153='3',
cust_attr154='3',
cust_attr155='2',
cust_attr156='1',
cust_attr157='6',
cust_attr158='6',
cust_attr159='7',
cust_attr160='7',
cust_attr161='1',
cust_attr162='1',
cust_attr163='3',
cust_attr164='7',
cust_attr165='3',
cust_attr166='3',
cust_attr167='2',
cust_attr168='1',
cust_attr169='6',
cust_attr170='6',
cust_attr171='7',
cust_attr172='7',
cust_attr173='1',
cust_attr174='1',
cust_attr175='3',
cust_attr176='7',
cust_attr177='3',
cust_attr178='3',
cust_attr179='2',
cust_attr180='1',
cust_attr181='6',
cust_attr182='6',
cust_attr183='7',
cust_attr184='7',
cust_attr185='1',
cust_attr186='1',
cust_attr187='3',
cust_attr188='7',
cust_attr189='3',
cust_attr190='3',
cust_attr191='2',
cust_attr192='1',
cust_attr193='6',
cust_attr194='6',
cust_attr195='7',
cust_attr196='7',
cust_attr197='1',
cust_attr198='1',
cust_attr199='3',
cust_attr200='7',
cust_attr201='3',
cust_attr202='3',
cust_attr203='2',
cust_attr204='1',
cust_attr205='6',
cust_attr206='6',
cust_attr207='7',
cust_attr208='7',
cust_attr209='1',
cust_attr210='1',
cust_attr211='3',
cust_attr212='7',
cust_attr213='3',
cust_attr214='3',
cust_attr215='2',
cust_attr216='1',
cust_attr217='6',
cust_attr218='6',
cust_attr219='7',
cust_attr220='7',
cust_attr221='1',
cust_attr222='1',
cust_attr223='3',
cust_attr224='7',
cust_attr225='3',
cust_attr226='3',
cust_attr227='2',
cust_attr228='1',
cust_attr229='6',
cust_attr230='6',
cust_attr231='7',
cust_attr232='7',
cust_attr233='1',
cust_attr234='1',
cust_attr235='3',
cust_attr236='7',
cust_attr237='3',
cust_attr238='3',
cust_attr239='2',
cust_attr240='1',
cust_attr241='6',
cust_attr242='6',
cust_attr243='7',
cust_attr244='7',
cust_attr245='1',
cust_attr246='1',
cust_attr247='3',
cust_attr248='7',
cust_attr249='3',
cust_attr250='3',
cust_attr251='2',
cust_attr252='1',
cust_attr253='6',
cust_attr254='6',
cust_attr255='7',
cust_attr256='7',
cust_attr257='1',
cust_attr258='1',
cust_attr259='3',
cust_attr260='7',
cust_attr261='3',
cust_attr262='3',
cust_attr263='2',
cust_attr264='1',
cust_attr265='6',
cust_attr266='6',
cust_attr267='7',
cust_attr268='7',
cust_attr269='1',
cust_attr270='1',
cust_attr271='3',
cust_attr272='7',
cust_attr273='3',
cust_attr274='3',
cust_attr275='2',
cust_attr276='1',
cust_attr277='6',
cust_attr278='6',
cust_attr279='7',
cust_attr280='7',
cust_attr281='1',
cust_attr282='1',
cust_attr283='3',
cust_attr284='7',
cust_attr285='3',
cust_attr286='3',
cust_attr287='2',
cust_attr288='1',
cust_attr289='6',
cust_attr290='1',
cust_attr291='1',
cust_attr292='1',
cust_attr293='7',
cust_attr294='7',
cust_attr295='7',
cust_attr296='7',
cust_attr297='7',
cust_attr298='7',
cust_attr299='7',
cust_attr300='7',
cust_attr301='7',
cust_attr302='7',
cust_attr303='7',
cust_attr304='7',
cust_attr305='7',
cust_attr306='7',
cust_attr307='7',
cust_attr308='7',
cust_attr309='7',
cust_attr310='7',
cust_attr311='7',
cust_attr312='7',
cust_attr313='7',
cust_attr314='7',
cust_attr315='7',
cust_attr316='7',
cust_attr317='7',
cust_attr318='7',
cust_attr319='7',
cust_attr320='7',
cust_attr321='7',
cust_attr322='7',
cust_attr323='7',
cust_attr324='7',
cust_attr325='7',
cust_attr326='7',
cust_attr327='7',
cust_attr328='7',
cust_attr329='7',
cust_attr330='7',
cust_attr331='7',
cust_attr332='7',
cust_attr333='7',
cust_attr334='7',
cust_attr335='7',
cust_attr336='7',
cust_attr337='7',
cust_attr338='7',
cust_attr339='7',
cust_attr340='7',
cust_attr341='1',
crtr_attr1='2',
crtr_attr2='2',
crtr_attr3='2',
crtr_attr4='2',
crtr_attr5='2',
crtr_attr6='2',
crtr_attr7='2',
crtr_attr8='2',
crtr_attr9='2',
crtr_attr10='2',
crtr_attr11='2',
crtr_attr12='2',
crtr_attr13='2',
crtr_attr14='2',
crtr_attr15='2',
crtr_attr16='2',
crtr_attr17='2',
crtr_attr18='2',
crtr_attr19='2',
crtr_attr20='2',
crtr_attr21='2',
crtr_attr22='2',
crtr_attr23='2',
crtr_attr24='2',
crtr_attr25='2',
crtr_attr26='2',
crtr_attr27='2',
crtr_attr28='2',
crtr_attr29='2',
crtr_attr30='2',
crtr_attr31='2',
crtr_attr32='2',
crtr_attr33='2',
crtr_attr34='2',
crtr_attr35='2',
crtr_attr36='2',
crtr_attr37='2',
crtr_attr38='2',
crtr_attr39='2',
crtr_attr40='2',
crtr_attr41='2',
crtr_attr42='2',
crtr_attr43='2',
crtr_attr44='2',
crtr_attr45='2',
crtr_attr46='2',
crtr_attr47='2',
crtr_attr48='2',
crtr_attr49='2',
crtr_attr50='2',
crtr_attr51='2',
crtr_attr52='2',
crtr_attr53='2',
crtr_attr54='2',
crtr_attr55='2',
crtr_attr56='2',
crtr_attr57='2',
crtr_attr58='2',
crtr_attr59='2',
crtr_attr60='2',
crtr_attr61='2',
crtr_attr62='2',
crtr_attr63='2',
crtr_attr64='2',
crtr_attr65='2',
crtr_attr66='2',
crtr_attr67='2',
crtr_attr68='2',
crtr_attr69='2',
crtr_attr70='2',
crtr_attr71='2',
crtr_attr72='2',
crtr_attr73='2',
crtr_attr74='2',
crtr_attr75='2',
crtr_attr76='2',
crtr_attr77='2',
crtr_attr78='2',
crtr_attr79='2',
crtr_attr80='2',
crtr_attr81='2',
crtr_attr82='2',
crtr_attr83='2',
crtr_attr84='2',
crtr_attr85='2',
crtr_attr86='2',
crtr_attr87='2',
crtr_attr88='2',
crtr_attr89='7',
crtr_attr90='7',
crtr_attr91='7',
crtr_attr92='7',
crtr_attr93='7',
crtr_attr94='7',
crtr_attr95='7',
crtr_attr96='7',
crtr_attr97='7',
crtr_attr98='7',
crtr_attr99='7',
crtr_attr100='7',
crtr_attr101='7',
crtr_attr102='7',
crtr_attr103='7',
crtr_attr104='7',
crtr_attr105='7',
crtr_attr106='7',
crtr_attr107='7',
crtr_attr108='7',
crtr_attr109='7',
crtr_attr110='7',
crtr_attr111='7',
crtr_attr112='7',
crtr_attr113='7',
crtr_attr114='7',
crtr_attr115='7',
crtr_attr116='7',
crtr_attr117='7',
crtr_attr118='7',
crtr_attr119='7',
crtr_attr120='7',
crtr_attr121='7',
crtr_attr122='7',
crtr_attr123='7',
crtr_attr124='7',
crtr_attr125='7',
crtr_attr126='7',
crtr_attr127='7',
crtr_attr128='7',
crtr_attr129='7',
crtr_attr130='7',
crtr_attr131='7',
crtr_attr132='7',
crtr_attr133='7',
crtr_attr134='7',
crtr_attr135='7',
crtr_attr136='7',
crtr_attr137='7',
crtr_attr138='7',
crtr_attr139='7',
crtr_attr140='7',
crtr_attr141='7',
crtr_attr142='7',
crtr_attr143='7',
crtr_attr144='7',
crtr_attr145='7',
crtr_attr146='7',
crtr_attr147='7',
crtr_attr148='7',
crtr_attr149='7',
crtr_attr150='7',
crtr_attr151='7',
crtr_attr152='7',
crtr_attr153='7',
crtr_attr154='7',
crtr_attr155='7',
crtr_attr156='7',
crtr_attr157='7',
crtr_attr158='7',
crtr_attr159='7',
crtr_attr160='7',
crtr_attr161='7',
crtr_attr162='7',
crtr_attr163='7',
crtr_attr164='7',
crtr_attr165='7',
crtr_attr166='7',
crtr_attr167='7',
crtr_attr168='7',
crtr_attr169='7',
crtr_attr170='7',
crtr_attr171='7',
crtr_attr172='7',
crtr_attr173='7',
crtr_attr174='7',
crtr_attr175='7',
crtr_attr176='7',
crtr_attr177='7',
crtr_attr178='7',
crtr_attr179='7',
ctr_attr180='7',
crtr_attr181='7',
crtr_attr182='7',
crtr_attr183='7',
crtr_attr184='7',
crtr_attr185='7',
crtr_attr186='7',
crtr_attr187='7',unused13='43',
cma_attr3746='2',unused14='51',
cma_attr3000='2',
cma_attr3001='2',
cma_attr3002='2',
cma_attr3100='2',
cma_attr3102='2',
cma_attr3104='2',
cma_attr3105='2',
cma_attr3109='2',
cma_attr3111='4',
cma_attr3113='4',
cma_attr3116='4',
cma_attr3117='4',
cma_attr3119='4',
cma_attr3120='4',
cma_attr3122='4',
cma_attr3123='4',
cma_attr3124='4',
cma_attr3126='4',
cma_attr3127='4',
cma_attr3131='4',
cma_attr3132='4',
cma_attr3133='2',
cma_attr3134='2',
cma_attr3135='2',
cma_attr3136='2',
cma_attr3137='2',
cma_attr3138='2',
cma_attr3139='2',
cma_attr3140='2',
cma_attr3141='2',
cma_attr3142='2',
cma_attr3143='2',
cma_attr3144='2',
cma_attr3145='2',
cma_attr3146='2',
cma_attr3147='2',
cma_attr3148='2',
cma_attr3150='2',
cma_attr3152='2',
cma_attr3153='2',
cma_attr3157='2',
cma_attr3159='7',
cma_attr3160='7',
cma_attr3161='7',
cma_attr3162='7',
cma_attr3163='7',
cma_attr3164='7',
cma_attr3165='7',
cma_attr3166='7',
cma_attr3167='7',
cma_attr3168='7',
cma_attr3169='7',
cma_attr3203='7',
cma_attr3204='7',
cma_attr3205='7',
cma_attr3206='7',
cma_attr3207='7',
cma_attr3208='7',
cma_attr3215='2',
cma_attr3217='2',
cma_attr3219='2',
cma_attr3223='2',
cma_attr3224='2',
cma_attr3228='7',
cma_attr3231='7',
cma_attr3234='7',
cma_attr3235='7',
cma_attr3236='7',
cma_attr3237='7',
cma_attr3239='7',
cma_attr3242='7',
cma_attr3245='7',
cma_attr3246='7',
cma_attr3257='2',
cma_attr3266='2',
cma_attr3268='2',
cma_attr3270='2',
cma_attr3272='2',
cma_attr3273='2',
cma_attr3276='2',
cma_attr3277='2',
cma_attr3285='2',
cma_attr3286='2',
cma_attr3288='2',
cma_attr3290='2',
cma_attr3292='2',
cma_attr3293='2',
cma_attr3296='2',
cma_attr3297='2',
cma_attr3307='2',
cma_attr3309='2',
cma_attr3311='2',
cma_attr3312='2',
cma_attr3315='2',
cma_attr3316='2',
cma_attr3318='2',
cma_attr3320='2',
cma_attr3322='2',
cma_attr3323='2',
cma_attr3326='2',
cma_attr3327='2',
cma_attr3329='2',
cma_attr3330='2',
cma_attr3332='2',
cma_attr3333='2',
cma_attr3335='2',
cma_attr3336='2',
cma_attr3338='2',
cma_attr3340='2',
cma_attr3347='2',
cma_attr3368='2',
cma_attr3370='2',
cma_attr3376='2',
cma_attr3379='2',
cma_attr3380='2',
cma_attr3382='2',
cma_attr3383='2',
cma_attr3385='2',
cma_attr3386='2',
cma_attr3387='2',
cma_attr3388='2',
cma_attr3389='2',
cma_attr3391='2',
cma_attr3392='2',
cma_attr3394='2',
cma_attr3395='2',
cma_attr3396='2',
cma_attr3397='2',
cma_attr3398='2',
cma_attr3400='2',
cma_attr3401='2',
cma_attr3403='2',
cma_attr3404='2',
cma_attr3405='2',
cma_attr3406='2',
cma_attr3407='2',
cma_attr3409='2',
cma_attr3410='2',
cma_attr3412='2',
cma_attr3413='2',
cma_attr3414='2',
cma_attr3415='2',
cma_attr3417='2',
cma_attr3419='2',
cma_attr3420='2',
cma_attr3422='2',
cma_attr3423='2',
cma_attr3424='2',
cma_attr3425='2',
cma_attr3426='2',
cma_attr3428='2',
cma_attr3430='2',
cma_attr3431='2',
cma_attr3433='2',
cma_attr3434='2',
cma_attr3435='2',
cma_attr3436='2',
cma_attr3437='2',
cma_attr3439='2',
cma_attr3441='2',
cma_attr3442='2',
cma_attr3444='2',
cma_attr3445='2',
cma_attr3446='2',
cma_attr3447='2',
cma_attr3448='2',
cma_attr3450='2',
cma_attr3452='2',
cma_attr3453='2',
cma_attr3455='2',
cma_attr3456='2',
cma_attr3457='2',
cma_attr3458='2',
cma_attr3535='2',
cma_attr3536='2',
cma_attr3537='2',
cma_attr3539='2',
cma_attr3540='2',
cma_attr3544='2',
cma_attr3546='2',
cma_attr3547='2',
cma_attr3548='2',
cma_attr3550='2',
cma_attr3551='2',
cma_attr3555='2',
cma_attr3557='2',
cma_attr3558='2',
cma_attr3559='2',
cma_attr3561='2',
cma_attr3562='2',
cma_attr3566='2',
cma_attr3568='2',
cma_attr3569='2',
cma_attr3570='2',
cma_attr3572='2',
cma_attr3573='2',
cma_attr3577='2',
cma_attr3579='2',
cma_attr3580='2',
cma_attr3592='2',
cma_attr3593='2',
cma_attr3594='2',
cma_attr3596='2',
cma_attr3597='2',
cma_attr3598='2',
cma_attr3600='2',
cma_attr3601='2',
cma_attr3602='2',
cma_attr3603='2',
cma_attr3605='2',
cma_attr3607='2',
cma_attr3608='2',
cma_attr3611='2',
cma_attr3612='2',
cma_attr3723='2',
cma_attr3724='2',
cma_attr3725='2',
cma_attr3726='2',
cma_attr3727='2',
cma_attr3728='2',
cma_attr3729='2',
cma_attr3900='5',
cma_attr3901='5',
cma_attr3902='1',
cma_attr3903='1',
cma_attr3904='1',
cma_attr3905='1',
cma_attr3907='2',
cma_attr3908='2',
cma_attr3909='2',
cma_attr3911='7',
cma_attr3912='7',
cma_attr3913='7',
cma_attr3747='2',
cma_attr3748='2',
cma_attr3749='2',
cma_attr3750='2',
cma_attr3751='2',
cma_attr3752='2',
cma_attr3753='2',
cma_attr3754='2',
cma_attr3812='4',
cma_attr3813='4',
cma_attr3993='5',
cma_attr3994='5',
cma_attr3995='5',
cma_attr3825='5',
cma_attr3826='5',
cma_attr3827='5',
cma_attr3828='5',
cma_attr3829='5',
cma_attr3830='5',
cma_attr3831='5',
cma_attr3832='5',
cma_attr3833='5',
cma_attr3834='5',
cma_attr3835='5',
cma_attr3836='5',
cma_attr3837='5',
cma_attr3838='5',
cma_attr3839='5',
cma_attr3840='5',
cma_attr3841='5',
cma_attr3842='5',
cma_attr3843='5',
cma_attr3844='5',
cma_attr3845='5',
cma_attr3854='5',
cma_attr3855='5',
cma_attr3856='5',
cma_attr3857='5',
cma_attr3858='5',
cma_attr3859='5',
cma_attr3860='5',
cma_attr3861='5',
cma_attr3862='5',
cma_attr3863='5',
cma_attr3864='5',
cma_attr3865='5',
cma_attr3866='5',
cma_attr3867='5',
cma_attr3868='5',
cma_attr3869='5',
cma_attr3938='5',
cma_attr3939='5',
cma_attr3940='5',
cma_attr3870='2',
cma_attr3871='2',
cma_attr3872='2',
cma_attr3873='2',
cma_attr3874='2',
cma_attr3875='2',
cma_attr3876='2',
cma_attr3877='2',
cma_attr3878='2',
cma_attr3879='2',
cma_attr3880='2',
cma_attr3881='2',
cma_attr3882='2',
cma_attr3883='2',
cma_attr3884='2',
cma_attr3885='2',
cma_attr3917='2',
cma_attr3918='2',
cma_attr3919='2',
cma_attr3920='2',
cma_attr3921='2',
cma_attr3922='2',
cma_attr3923='2',
cma_attr3924='2',
cma_attr3925='2',
cma_attr3926='2',
cma_attr3927='2',
cma_attr3928='2',
cma_attr3929='2',
cma_attr3930='2',
cma_attr3931='2',
cma_attr3932='2',
cma_attr3933='2',
cma_attr3886='2',
cma_attr3934='2',
cma_attr3887='2',
cma_attr3894='5',
cma_attr3895='5',
cma_attr3896='5',
cma_attr3897='5',
cma_attr3898='5',
cma_attr3899='5',
cma_attr3914='5',
cma_attr3915='5',
cma_attr3916='5',
cma_attr3941='5',
cma_attr3942='5',
cma_attr3943='5',
cma_attr3944='5',
cma_attr3945='5',
cma_attr3946='5',
cma_attr3947='5',
cma_attr3948='5',
cma_attr3949='5',
cma_attr3950='2',
cma_attr3951='2',
cma_attr3952='2',
cma_attr3953='2',
cma_attr3954='2',
cma_attr3955='2',
cma_attr3956='2',
cma_attr3957='2',
cma_attr3958='2',
cma_attr3959='2',
cma_attr3960='2',
cma_attr3961='2',
cma_attr3962='2',
cma_attr3963='2',
cma_attr3964='2',
cma_attr3968='5',
cma_attr3969='5',
cma_attr3970='5',
cma_attr3971='5',
cma_attr3973='5',
cma_attr3974='5',
cma_attr3975='5',
cma_attr3976='5',
cma_attr3978='5',
cma_attr3979='5',
cma_attr3980='5',
cma_attr3981='5',
cma_attr3983='5',
cma_attr3984='5',
cma_attr3985='5',
cma_attr3986='5',
cma_attr3987='5',
unused15='1',
census_tract_2010='6',
unused16='1',
census_block_2010='4',
unused17='1',
county_code_2010='3',
unused18='1',
preserve_blanks=on,
NULL=' ',
line_delim=E'\n'
)
;
drop external table if exists large_custom_format_definitions;
